/*
TCP-Server IR-Module
Change SSID and PASSWORD and irCom1-irCom6.
*/

#define SSID "[Your SSID]"
#define PASSWORD "[Your Password]"

#define irCom1 0x20DF10EF      //POWER
#define irCom2 0x20DF00FF      //Ch+
#define irCom3 0x20DF807F      //Ch-
#define irCom4 0x20DF22DD      //OK
#define irCom5 0x20DF40BF      //Vol+
#define irCom6 0x20DFC03F      //Vol-

#define LED_WLAN 13

#define RED 3
#define GREEN 5
#define BLUE 6
#define GND 4

#define RECV_PIN  7
#define GND2  8
#define VCC  9

#define DEBUG true

#include <IRremote.h>

#include <SoftwareSerial.h>
SoftwareSerial esp8266(11, 12); // RX, TX

const byte site[] PROGMEM = {
  60, 72, 84, 77, 76, 62, 10, 9, 60, 72, 69, 65, 68, 62, 10, 9, 9, 60, 116, 105, 116, 108, 101, 62, 10, 9, 9, 67, 111, 108, 111, 114, 32, 82, 71, 66, 10, 9, 9, 60, 47, 116, 105, 116, 108, 101, 62, 10, 9, 60, 47, 72, 69, 65, 68, 62, 10, 10, 60, 66, 79, 68, 89, 32, 98, 103, 99, 111, 108, 111, 114, 61, 34, 42, 99, 111, 108, 111, 114, 42, 34, 32, 116, 101, 120, 116, 61, 34, 35, 48, 48, 48, 48, 48, 48, 34, 62, 10, 60, 70, 79, 78, 84, 32, 115, 105, 122, 101, 61, 34, 54, 34, 32, 70, 65, 67, 69, 61, 34, 86, 101, 114, 100, 97, 110, 97, 34, 62, 10, 9, 67, 111, 108, 111, 114, 32, 82, 71, 66, 10, 60, 47, 70, 79, 78, 84, 62, 10, 60, 70, 79, 78, 84, 32, 115, 105, 122, 101, 61, 34, 51, 34, 32, 70, 65, 67, 69, 61, 34, 86, 101, 114, 100, 97, 110, 97, 34, 62, 10, 60, 72, 82, 62, 10, 67, 117, 114, 114, 101, 110, 116, 32, 67, 111, 108, 111, 114, 10, 60, 47, 102, 111, 110, 116, 62, 10, 60, 47, 72, 84, 77, 76, 62, 10, 10
};

byte r, g, b;

IRrecv irrecv(RECV_PIN);
decode_results results;

void setup() {
  Serial.begin(19200);
  esp8266.begin(19200);

  pinMode(GND, OUTPUT);
  pinMode(GND2, OUTPUT);
  pinMode(VCC, OUTPUT);

  pinMode(RED, OUTPUT);
  pinMode(GREEN, OUTPUT);
  pinMode(BLUE, OUTPUT);

  digitalWrite (GND, LOW);
  digitalWrite (GND2, LOW);
  digitalWrite (VCC, HIGH);
  
  if (!espConfig()) serialDebug();
  else digitalWrite(LED_WLAN, HIGH);

  if (configTCPServer())  debug("Server Aktiv"); else debug("Server Error");

  irrecv.enableIRIn(); // Start the receiver
}

void loop() {
  irAnalyse(); //Check incomming IR

  if (esp8266.available()) // check if the esp is sending a message
  {
    if (esp8266.find("+IPD,"))
    {
      debug("Incomming Request");
      int connectionId = esp8266.parseInt();

      if (sendWebsite(connectionId, createWebsite())) debug("Website send OK"); else debug("Website send Error");
    }
  }
}

void irAnalyse()
{
  if (irrecv.decode(&results)) {
    Serial.print("Value ");
    Serial.println (results.value, HEX);
    Serial.println("Type:" + String(results.decode_type));

    switch (results.value)
    {
      case irCom1:  
        r = 255; g = 255; b = 255;
        break;

      case irCom2: 
        r = 255; g = 0; b = 0;
        break;
        
      case  irCom3: 
        r = 0; g = 255; b = 0;
        break;

      case irCom4: 
        r = 0; g = 0; b = 0;
        break;

      case irCom5:  
        r = 0; g = 0; b = 255;
        break;

      case irCom6: 
        r = 255; g = 255; b = 0;
        break;
    }
    analogWrite(RED, r);
    analogWrite(GREEN, g);
    analogWrite(BLUE, b);
    irrecv.resume(); // Receive the next value
  }

}

String rgbToHex(byte r, byte g, byte b) {
  String hex = "#";
  if (r == 0) hex += "00";
  else if (r <= 0x0F)  hex += "0" + String(r, HEX);
  else hex += String(r, HEX);

  if (g == 0) hex += "00";
  else if (g <= 0x0F)  hex += "0" + String(g, HEX);
  else hex += String(g, HEX);

  if (b == 0) hex += "00";
  else if (b <= 0x0F)  hex += "0" + String(b, HEX);
  else hex += String(b, HEX);

  return hex;
}

boolean sendWebsite(int connectionId, String webpage)
{
  boolean succes = true;

  if (sendCom("AT+CIPSEND=" + String(connectionId) + "," + String(webpage.length()), ">"))
  {
    esp8266.print(webpage);
    esp8266.find("SEND OK");
    succes &= sendCom("AT+CIPCLOSE=" + String(connectionId), "OK");
  }
  else
  {
    succes = false;
  }
  return succes;
}

String createWebsite()
{
  String xBuffer;

  for (int i = 0; i <= sizeof(site); i++)
  {
    char myChar = pgm_read_byte_near(site + i);
    xBuffer += myChar;
  }

  xBuffer.replace("*color*", rgbToHex(r, g, b));

  return xBuffer;
}


//-----------------------------------------Config ESP8266------------------------------------

boolean espConfig()
{
  boolean succes = true;
  esp8266.setTimeout(5000);
  succes &= sendCom("AT+RST", "ready");
  esp8266.setTimeout(1000);
  if (configStation(SSID, PASSWORD)) {
    succes &= true;
    debug("WLAN Connected");
    debug("My IP is:");
    debug(sendCom("AT+CIFSR"));
  }
  else
  {
    succes &= false;
  }
  //shorter Timeout for faster wrong UPD-Comands handling
  succes &= sendCom("AT+CIPMODE=0", "OK");
  succes &= sendCom("AT+CIPMUX=0", "OK");

  return succes;
}

boolean configTCPServer()
{
  boolean succes = true;

  succes &= (sendCom("AT+CIPMUX=1", "OK"));
  succes &= (sendCom("AT+CIPSERVER=1,80", "OK"));

  return succes;

}

boolean configTCPClient()
{
  boolean succes = true;

  succes &= (sendCom("AT+CIPMUX=0", "OK"));
  //succes &= (sendCom("AT+CIPSERVER=1,80", "OK"));

  return succes;

}


boolean configStation(String vSSID, String vPASSWORT)
{
  boolean succes = true;
  succes &= (sendCom("AT+CWMODE=1", "OK"));
  esp8266.setTimeout(20000);
  succes &= (sendCom("AT+CWJAP=\"" + String(vSSID) + "\",\"" + String(vPASSWORT) + "\"", "OK"));
  esp8266.setTimeout(1000);
  return succes;
}

boolean configAP()
{
  boolean succes = true;

  succes &= (sendCom("AT+CWMODE=2", "OK"));
  succes &= (sendCom("AT+CWSAP=\"NanoESP\",\"\",5,0", "OK"));

  return succes;
}

boolean configUDP()
{
  boolean succes = true;

  succes &= (sendCom("AT+CIPMODE=0", "OK"));
  succes &= (sendCom("AT+CIPMUX=0", "OK"));
  succes &= sendCom("AT+CIPSTART=\"UDP\",\"192.168.255.255\",90,91,2", "OK"); //Importand Boradcast...Reconnect IP
  return succes;
}




//-----------------------------------------------Controll ESP-----------------------------------------------------

boolean sendUDP(String Msg)
{
  boolean succes = true;

  succes &= sendCom("AT+CIPSEND=" + String(Msg.length() + 2), ">");    //+",\"192.168.4.2\",90", ">");
  if (succes)
  {
    succes &= sendCom(Msg, "OK");
  }
  return succes;
}


boolean sendCom(String command, char respond[])
{
  esp8266.println(command);
  if (esp8266.findUntil(respond, "ERROR"))
  {
    return true;
  }
  else
  {
    debug("ESP SEND ERROR: " + command);
    return false;
  }
}

String sendCom(String command)
{
  esp8266.println(command);
  return esp8266.readString();
}



//-------------------------------------------------Debug Functions------------------------------------------------------
void serialDebug() {
  while (true)
  {
    if (esp8266.available())
      Serial.write(esp8266.read());
    if (Serial.available())
      esp8266.write(Serial.read());
  }
}

void debug(String Msg)
{
  if (DEBUG)
  {
    Serial.println(Msg);
  }
}
